@@ -1,19 +1,20 @@
 /*
- *  Copyright (c) Stephan Arts 2006-2011 <stephan@xfce.org>
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  Copyright (c) Stephan Arts 2006-2012 <stephan@xfce.org>
  *
+ *  This program is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU General Public License
+ *  as published by the Free Software Foundation; either version 2
+ *  of the License, or (at your option) any later version.
+ * 
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
- *
+ *  GNU General Public License for more details.
+ * 
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 #include <config.h>
@@ -80,6 +81,8 @@ struct _RsttoImageViewerPriv
     GdkColor                    *bg_color;
     GdkColor                    *bg_color_fs;
 
+    gboolean                     limit_quality;
+
     GError                      *error;
 
     RsttoImageViewerTransaction *transaction;
@@ -198,6 +201,11 @@ rstto_image_viewer_get_property (
         GValue     *value,
         GParamSpec *pspec);
 
+static void
+cb_rstto_image_viewer_file_changed (
+        RsttoFile        *r_file,
+        RsttoImageViewer *viewer );
+
 static void
 rstto_image_viewer_set_motion_state (RsttoImageViewer *viewer, RsttoImageViewerMotionState state);
 
@@ -248,6 +256,11 @@ rstto_popup_menu (
         GtkWidget *widget);
 
 static void
+cb_rstto_limit_quality_changed (
+        GObject *settings,
+        GParamSpec *pspec,
+        gpointer user_data);
+static void
 cb_rstto_bgcolor_changed (
         GObject *settings,
         GParamSpec *pspec,
@@ -342,6 +355,12 @@ rstto_image_viewer_init ( GObject *object )
             "notify::bgcolor-override",
             G_CALLBACK (cb_rstto_bgcolor_changed),
             viewer);
+    g_signal_connect (
+            G_OBJECT(viewer->priv->settings),
+            "notify::limit-quality",
+            G_CALLBACK (cb_rstto_limit_quality_changed),
+            viewer);
+
     g_signal_connect (
             G_OBJECT(viewer->priv->settings),
             "notify::revert-zoom-direction",
@@ -429,6 +448,13 @@ rstto_image_viewer_class_init(RsttoImageViewerClass *viewer_class)
             NULL, NULL,
             g_cclosure_marshal_VOID__VOID,
             G_TYPE_NONE, 0);
+    g_signal_new ("status-changed",
+            G_TYPE_FROM_CLASS (object_class),
+            G_SIGNAL_RUN_FIRST,
+            0,
+            NULL, NULL,
+            g_cclosure_marshal_VOID__VOID,
+            G_TYPE_NONE, 0);
     g_signal_new ("files-dnd",
             G_TYPE_FROM_CLASS (object_class),
             G_SIGNAL_RUN_FIRST,
@@ -462,6 +488,7 @@ rstto_image_viewer_realize(GtkWidget *widget)
     GValue val_bg_color = {0, };
     GValue val_bg_color_override = {0, };
     GValue val_bg_color_fs = {0, };
+    GValue val_limit_quality = {0, };
 
     GdkWindowAttr attributes;
     gint attributes_mask;
@@ -474,6 +501,7 @@ rstto_image_viewer_realize(GtkWidget *widget)
     g_value_init (&val_bg_color, GDK_TYPE_COLOR);
     g_value_init (&val_bg_color_fs, GDK_TYPE_COLOR);
     g_value_init (&val_bg_color_override, G_TYPE_BOOLEAN);
+    g_value_init (&val_limit_quality, G_TYPE_BOOLEAN);
 
     attributes.x = widget->allocation.x;
     attributes.y = widget->allocation.y;
@@ -506,6 +534,12 @@ rstto_image_viewer_realize(GtkWidget *widget)
             G_OBJECT(viewer->priv->settings),
             "bgcolor-fullscreen",
             &val_bg_color_fs);
+    g_object_get_property (
+            G_OBJECT(viewer->priv->settings),
+            "limit-quality",
+            &val_limit_quality);
+
+    viewer->priv->limit_quality = g_value_get_boolean (&val_limit_quality);
 
     if (TRUE == g_value_get_boolean (&val_bg_color_override))
     {
@@ -636,6 +670,11 @@ rstto_image_viewer_destroy(GtkObject *object)
             g_object_unref (viewer->priv->pixbuf);
             viewer->priv->pixbuf = NULL;
         }
+        if (viewer->priv->iter)
+        {
+            g_object_unref (viewer->priv->iter);
+            viewer->priv->iter = NULL;
+        }
         g_free (viewer->priv);
         viewer->priv = NULL;
     }
@@ -919,9 +958,30 @@ correct_adjustments ( RsttoImageViewer *viewer )
     gdouble scale = viewer->priv->scale;
 
 
+    /* Check if the image-size makes sense,
+     * if not, set the upper limits to 0.0
+     */
+    if ( (image_width <= 1.0) || (image_height <= 1.0) )
+    {
+        gtk_adjustment_set_value (
+                viewer->hadjustment,
+                0.0 );
+        gtk_adjustment_set_upper (
+                viewer->hadjustment,
+                0.0 );
+        gtk_adjustment_set_value (
+                viewer->vadjustment,
+                0.0 );
+        gtk_adjustment_set_upper (
+                viewer->vadjustment,
+                0.0 );
+        return;
+    }
+
     g_object_freeze_notify(G_OBJECT(viewer->hadjustment));
     g_object_freeze_notify(G_OBJECT(viewer->vadjustment));
 
+
     switch (viewer->priv->orientation)
     {
         case RSTTO_IMAGE_ORIENT_NONE:
@@ -1009,15 +1069,27 @@ paint_clock (
         cairo_t *ctx )
 {
     //RsttoImageViewer *viewer = RSTTO_IMAGE_VIEWER (widget);
-    gdouble width = 60;
-    gdouble height = 60;
-    gdouble offset = height*0.15;
+    gdouble width;
+    gdouble height;
+    gdouble offset;
     gint i = 0;
     time_t t = time(NULL);
     struct tm *lt = localtime(&t);
 
     gdouble hour_angle = (gdouble)(M_PI*2)/12*((gdouble)(lt->tm_hour%12+6)+((M_PI*2)/720.0*(gdouble)lt->tm_min));
 
+    if (widget->allocation.width < widget->allocation.height)
+    {
+        width = 40 + ((gdouble)widget->allocation.width * 0.07);
+    }
+    else
+    {
+        width = 40 + ((gdouble)widget->allocation.height * 0.07);
+    }
+
+    height = width;
+    offset = height * 0.15;
+
     cairo_save (ctx);
 
     cairo_translate (
@@ -1050,7 +1122,7 @@ paint_clock (
         cairo_arc (
             ctx,
             00, -1*((width/2)-5),
-            3,
+            width/20,
             0,
             2*M_PI );
         cairo_fill (ctx);
@@ -1059,7 +1131,7 @@ paint_clock (
 
 /***/
     cairo_save (ctx);
-    cairo_set_line_width (ctx, 5.0);
+    cairo_set_line_width (ctx, width/15);
     cairo_set_line_cap (ctx, CAIRO_LINE_CAP_ROUND);
     cairo_rotate (
         ctx,
@@ -1073,7 +1145,7 @@ paint_clock (
         ctx);
     cairo_restore (ctx);
 /***/
-    cairo_set_line_width (ctx, 5.0);
+    cairo_set_line_width (ctx, width/15);
     cairo_set_line_cap (ctx, CAIRO_LINE_CAP_ROUND);
     cairo_rotate (
         ctx,
@@ -1561,6 +1633,15 @@ rstto_image_viewer_set_file (
                 } 
 
                 g_object_ref (file);
+                g_signal_connect (
+                        file,
+                        "changed",
+                        G_CALLBACK (cb_rstto_image_viewer_file_changed),
+                        viewer);
+                g_signal_handlers_disconnect_by_func (
+                        viewer->priv->file,
+                        cb_rstto_image_viewer_file_changed,
+                        viewer );
                 g_object_unref (viewer->priv->file);
 
                 viewer->priv->file = file;
@@ -1581,6 +1662,11 @@ rstto_image_viewer_set_file (
         }
         else
         {
+            g_signal_connect (
+                    file,
+                    "changed",
+                    G_CALLBACK (cb_rstto_image_viewer_file_changed),
+                    viewer);
             g_object_ref (file);
             viewer->priv->file = file;
             rstto_image_viewer_load_image (viewer, viewer->priv->file, scale);
@@ -1613,31 +1699,16 @@ rstto_image_viewer_set_file (
         }
         if (viewer->priv->file)
         {
+            g_signal_handlers_disconnect_by_func (
+                    viewer->priv->file,
+                    cb_rstto_image_viewer_file_changed,
+                    viewer );
             g_object_unref (viewer->priv->file);
             viewer->priv->file = NULL;
 
-            g_object_freeze_notify(G_OBJECT(viewer->hadjustment));
-            g_object_freeze_notify(G_OBJECT(viewer->vadjustment));
-
-            gtk_adjustment_set_upper (
-                    viewer->hadjustment,
-                    0.0);
-            gtk_adjustment_set_value (
-                    viewer->hadjustment,
-                    0.0);
-
-            gtk_adjustment_set_upper (
-                    viewer->vadjustment,
-                    0.0);
-            gtk_adjustment_set_value (
-                    viewer->vadjustment,
-                    0.0);
-
-            /*
-             * Enable signals on the adjustments.
-             */
-            g_object_thaw_notify(G_OBJECT(viewer->vadjustment));
-            g_object_thaw_notify(G_OBJECT(viewer->hadjustment));
+            /* Reset the image-size to 0.0 */
+            viewer->priv->image_height = 0.0;
+            viewer->priv->image_width = 0.0;
             
             gdk_window_invalidate_rect (
                     widget->window,
@@ -1668,11 +1739,13 @@ rstto_image_viewer_load_image (
     }
 
     transaction->loader = gdk_pixbuf_loader_new_with_mime_type (rstto_file_get_content_type (file), NULL);
+
     /* HACK HACK HACK */
     if (transaction->loader == NULL)
     {
         transaction->loader = gdk_pixbuf_loader_new();
     }
+
     transaction->cancellable = g_cancellable_new();
     transaction->buffer = g_new0 (guchar, RSTTO_IMAGE_VIEWER_BUFFER_SIZE);
     transaction->file = file;
@@ -1714,7 +1787,9 @@ rstto_image_viewer_transaction_free (RsttoImageViewerTransaction *tr)
 }
 
 void
-rstto_image_viewer_set_scale (RsttoImageViewer *viewer, gdouble scale)
+rstto_image_viewer_set_scale (
+        RsttoImageViewer *viewer,
+        gdouble scale)
 {
     GtkWidget *widget = GTK_WIDGET (viewer);
 
@@ -1792,7 +1867,9 @@ rstto_image_viewer_get_scale (RsttoImageViewer *viewer)
 }
 
 static void
-rstto_image_viewer_set_motion_state (RsttoImageViewer *viewer, RsttoImageViewerMotionState state)
+rstto_image_viewer_set_motion_state (
+        RsttoImageViewer *viewer,
+        RsttoImageViewerMotionState state)
 {
     viewer->priv->motion.state = state;
 }
@@ -1828,13 +1905,15 @@ rstto_image_viewer_set_orientation (
 }
 
 RsttoImageOrientation
-rstto_image_viewer_get_orientation (RsttoImageViewer *viewer)
+rstto_image_viewer_get_orientation (
+        RsttoImageViewer *viewer)
 {
     return viewer->priv->orientation;
 }
 
 gint
-rstto_image_viewer_get_width (RsttoImageViewer *viewer)
+rstto_image_viewer_get_width (
+        RsttoImageViewer *viewer)
 {
     if (viewer)
     {
@@ -1843,7 +1922,8 @@ rstto_image_viewer_get_width (RsttoImageViewer *viewer)
     return 0;
 }
 gint
-rstto_image_viewer_get_height (RsttoImageViewer *viewer)
+rstto_image_viewer_get_height (
+        RsttoImageViewer *viewer)
 {
     if (viewer)
     {
@@ -1854,8 +1934,8 @@ rstto_image_viewer_get_height (RsttoImageViewer *viewer)
 
 void
 rstto_image_viewer_set_menu (
-    RsttoImageViewer *viewer,
-    GtkMenu *menu)
+        RsttoImageViewer *viewer,
+        GtkMenu *menu)
 {
     if (viewer->priv->menu)
     {
@@ -1881,7 +1961,9 @@ rstto_image_viewer_set_menu (
 /************************/
 
 static void
-cb_rstto_image_viewer_value_changed(GtkAdjustment *adjustment, RsttoImageViewer *viewer)
+cb_rstto_image_viewer_value_changed (
+        GtkAdjustment *adjustment,
+        RsttoImageViewer *viewer)
 {
     GtkWidget *widget = GTK_WIDGET (viewer);
     gdk_window_invalidate_rect (
@@ -1961,7 +2043,9 @@ cb_rstto_image_viewer_read_input_stream_ready (
 
 
 static void
-cb_rstto_image_loader_area_prepared (GdkPixbufLoader *loader, RsttoImageViewerTransaction *transaction)
+cb_rstto_image_loader_area_prepared (
+        GdkPixbufLoader *loader,
+        RsttoImageViewerTransaction *transaction)
 {
     gint timeout = 0;
     RsttoImageViewer *viewer = transaction->viewer;
@@ -2017,52 +2101,66 @@ cb_rstto_image_loader_size_prepared (
 {
     gint s_width = gdk_screen_get_width (default_screen);
     gint s_height = gdk_screen_get_height (default_screen);
+    gboolean limit_quality = transaction->viewer->priv->limit_quality;
+
+    /*
+     * By default, the image-size won't be limited to screen-size (since it's smaller)
+     * or, because we don't want to reduce it.
+     * Set the image_scale to 1.0 (100%)
+     */
+    transaction->image_scale = 1.0;
 
     transaction->image_width = width;
     transaction->image_height = height;
 
-    /*
-     * Set the maximum size of the loaded image to the screen-size.
-     * TODO: Add some 'smart-stuff' here
-     */
-    if (s_width < width || s_height < height)
+
+    if (limit_quality == TRUE)
     {
         /*
-         * The image is loaded at the screen_size, calculate how this fits best.
-         *  scale = MIN(width / screen_width, height / screen_height)
-         *
+         * Set the maximum size of the loaded image to the screen-size.
+         * TODO: Add some 'smart-stuff' here
          */
-        if(((gdouble)width / (gdouble)s_width) < ((gdouble)height / (gdouble)s_height))
+        if (s_width < width || s_height < height)
         {
-            transaction->image_scale = (gdouble)s_width / (gdouble)width;
-            gdk_pixbuf_loader_set_size (
-                    loader,
-                    s_width,
-                    (gint)((gdouble)height/(gdouble)width*(gdouble)s_width)); 
+            /*
+             * The image is loaded at the screen_size, calculate how this fits best.
+             *  scale = MIN(width / screen_width, height / screen_height)
+             *
+             */
+            if(((gdouble)width / (gdouble)s_width) < ((gdouble)height / (gdouble)s_height))
+            {
+                transaction->image_scale = (gdouble)s_width / (gdouble)width;
+                gdk_pixbuf_loader_set_size (
+                        loader,
+                        s_width,
+                        (gint)((gdouble)height/(gdouble)width*(gdouble)s_width)); 
+            }
+            else
+            {
+                transaction->image_scale = (gdouble)s_height / (gdouble)height;
+                gdk_pixbuf_loader_set_size (
+                        loader,
+                        (gint)((gdouble)width/(gdouble)height*(gdouble)s_height),
+                        s_height); 
+            }
         }
         else
         {
-            transaction->image_scale = (gdouble)s_height / (gdouble)height;
-            gdk_pixbuf_loader_set_size (
-                    loader,
-                    (gint)((gdouble)width/(gdouble)height*(gdouble)s_height),
-                    s_height); 
+            /*
+             * Image-size won't be limited to screen-size (since it's smaller)
+             * Set the image_scale to 1.0 (100%)
+             */
+            transaction->image_scale = 1.0;
         }
     }
-    else
-    {
-        /*
-         * Image-size won't be limited to screen-size (since it's smaller)
-         * Set the image_scale to 1.0 (100%)
-         */
-        transaction->image_scale = 1.0;
-    }
 
     transaction->orientation = rstto_file_get_orientation (transaction->file);
 }
 
 static void
-cb_rstto_image_loader_closed (GdkPixbufLoader *loader, RsttoImageViewerTransaction *transaction)
+cb_rstto_image_loader_closed (
+        GdkPixbufLoader *loader,
+        RsttoImageViewerTransaction *transaction)
 {
     RsttoImageViewer *viewer = transaction->viewer;
     GtkWidget *widget = GTK_WIDGET(viewer);
@@ -2491,13 +2589,14 @@ rstto_button_press_event (
         if (viewer->priv->menu)
         {
             gtk_widget_show_all(GTK_WIDGET(viewer->priv->menu));
-            gtk_menu_popup(viewer->priv->menu,
-                           NULL,
-                           NULL,
-                           NULL,
-                           NULL,
-                           3,
-                           event->time);
+            gtk_menu_popup (
+                    viewer->priv->menu,
+                    NULL,
+                    NULL,
+                    NULL,
+                    NULL,
+                    3,
+                    event->time);
         }
         return TRUE;
     }
@@ -2675,6 +2774,32 @@ rstto_button_release_event (
     return FALSE;
 }
 
+static void
+cb_rstto_limit_quality_changed (
+        GObject *settings,
+        GParamSpec *pspec,
+        gpointer user_data)
+{
+    RsttoImageViewer *viewer = RSTTO_IMAGE_VIEWER (user_data);
+
+    GValue val_limit_quality = {0, };
+
+    g_value_init (&val_limit_quality, G_TYPE_BOOLEAN);
+
+    g_object_get_property (
+            G_OBJECT(viewer->priv->settings),
+            "limit-quality",
+            &val_limit_quality);
+
+    viewer->priv->limit_quality = g_value_get_boolean (
+            &val_limit_quality);
+
+    rstto_image_viewer_load_image (
+            viewer,
+            viewer->priv->file,
+            viewer->priv->scale);
+}
+
 static void
 cb_rstto_bgcolor_changed (
         GObject *settings,
@@ -2753,8 +2878,15 @@ rstto_popup_menu (
 }
 
 static void
-cb_rstto_image_viewer_dnd (GtkWidget *widget, GdkDragContext *context, gint x, gint y, GtkSelectionData *data,
-              guint info, guint time_, RsttoImageViewer *viewer)
+cb_rstto_image_viewer_dnd (
+        GtkWidget *widget,
+        GdkDragContext *context,
+        gint x,
+        gint y,
+        GtkSelectionData *data,
+        guint info,
+        guint time_,
+        RsttoImageViewer *viewer )
 {
     g_return_if_fail ( RSTTO_IS_IMAGE_VIEWER(viewer) );
 
@@ -2855,3 +2987,26 @@ rstto_image_viewer_set_show_clock (
     }
 }
 
+gboolean
+rstto_image_viewer_is_busy (
+        RsttoImageViewer *viewer )
+{
+    if (viewer->priv->transaction != NULL)
+    {
+        return TRUE;
+    }
+    return FALSE;
+}
+
+static void
+cb_rstto_image_viewer_file_changed (
+        RsttoFile        *r_file,
+        RsttoImageViewer *viewer )
+{
+    rstto_image_viewer_load_image (
+            viewer,
+            r_file,
+            viewer->priv->scale);
+
+    g_signal_emit_by_name(viewer, "status-changed");
+}
